/*###########################################################################################
#
#  FILE AUTOMATICALLY GENERATED BY JFML FOR RUNNING A FUZZY SYSTEM IN AN ARDUINO BOARD 
#  YOU CAN EDIT THIS FILE ACCORDING TO YOUR SPECIFICATION
#
#    VARIABLES - SENSORS
#      1: temperature -- DHT22 temperature
#
#############################################################################################*/


/********** SENSOR LIBRARIES **************/
#include <DHT.h>  // http://www.electroschematics.com/wp-content/uploads/2015/02/DHT.rar
/******************************************/

/********** WIFI LIBRARIES **************/
#include <ESP8266WiFi.h>  // http://arduino.esp8266.com/stable/package_esp8266com_index.json
/******************************************/

 /********* WIFI VARIABLES ************/ 
const char * embeddedSystemName = "arduino_WIFI_DHT22TEMP";
const char * ssid = "SSID"; 
const char * password = "PASSWORD"; 
const char * host = "IP_ADDRESS";
const int port = 8080;
WiFiClient client;

/************ MAPPING VARIABLES ****************/
#define numVariables 1

#define pin_temperature 4
#define index_temperature 0
/*************************************************/

/********* COMMUNICATION PROTOCOL DEFINITIONS ************/
#define BAUD_RATE 9600
#define MAX_MESSAGE_SIZE 200 

String START_READ = "r"; 
String START_WRITE = "w"; 
String STOP = "x"; 
String ACK = "ACK"; 

boolean start_read=false; //boolean variable for start reading in Arduino 
boolean start_write=false; //boolean variable for start writing in Arduino 
boolean stop_communication=false; //boolean variable for stop communication with Arduino 

int read_function[numVariables]; 
int write_function[numVariables]; 
float write_value[numVariables]; 
float mapfloat(float x, float in_min, float in_max, float out_min, float out_max){
		return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}
/*********************************************************/

/********** TEMPERATURE / HUMIDITY SENSOR DHT22 ************/ 
#define DHTPIN 4// what pin we're connected to 
#define DHTTYPE DHT22  // Sensor type
DHT dht_temperature(DHTPIN, DHTTYPE); //// Initialize DHT sensor 
/***********************************************************/


void setup() { 
	 // Initialize serial 
	Serial.begin(BAUD_RATE); 

  // Setup ESP8266
  Serial.println(); 
  Serial.println(); 
  Serial.print("Connecting to "); 
  Serial.println(ssid); 

  //Initialize Wifi
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) { 
  	 delay(500); 
  	 Serial.print("."); 
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: "); 
  Serial.println(WiFi.localIP()); 

  //Inicialize Connection
  while (client.connect(host, port) == false) {
  	 Serial.println("connection to socket failed");
  	 delay(1000);
  }

  Serial.println("connection to socket... [OK]");
  //Send the name of the embedded system to indentify
  client.println(embeddedSystemName);

	// Pin mode
	pinMode(pin_temperature, INPUT); 

	// Testing setup code

	// Initialize libraries setup code
	dht_temperature.begin();
}

void loop() { 
	int numActions = processingActionPacket();

	// read from Arduino and send data to Java
	if(start_read && !stop_communication){
		readValuesFromArduino(numActions);
	}

	// write from Java to Arduino
	if(start_write && !stop_communication){ 
		writeValuesFromJava(numActions);
	}
}

int processingActionPacket(){
	boolean first_command = false;
	int num_actions = 0; // number of pins requested in the message

	while (client.available()) {
	char input[MAX_MESSAGE_SIZE];

		// Reading the bytes
		String str = client.readStringUntil('\r');
		client.flush();
		str.toCharArray(input, MAX_MESSAGE_SIZE);

		char * command;
		command = strtok (input,"&");
		while (command != NULL){
			String action[2];
			int i=0;
			char* p=command;
			char* str;
			while ((str = strtok_r(p, ":", &p)) != NULL)
				action[i++]=str;

			// if this is the first command
			 if(!first_command){
				if(action[0]==START_READ)
					start_read=true;

				if(action[0]==START_WRITE)
					start_write=true;

				if(action[0]==STOP)
					stop_communication=true;
				else
					stop_communication=false;

				first_command = true;
			}
			else{ //Reading the variable index and value
				if(start_read){
					read_function[num_actions++]=action[0].toInt();
				}
				if(start_write){
					write_function[num_actions]=action[0].toInt();
					write_value[num_actions++]=action[1].toFloat();
				}
			}
			command = strtok (NULL, "&");
		}
	}
	return num_actions;
}

void readValuesFromArduino(int num){
	String val = "";
	String message="";
	for(int i = 0; i < num; i++) {
		switch(read_function[i]){
			case index_temperature:
				val = read_temperature();
				break;
		}
		message += String(read_function[i]); // variable index in JFML
		message += "-";
		message += val; // value of the read
		if(i<num-1)
			message +="#";

		delay(10); // delay in between reads for stability
	}
	Serial.println(message);
	client.println(message);
	start_read = false;
}

void writeValuesFromJava(int num){
	String val = "";
	String message="";
	for(int i = 0; i < num; i++) {
		switch(write_function[i]){
		}
		message += String(write_function[i]); // variable index in JFML
		message += "-";
		message += val; // value of the write (ACK)
		if(i<num-1)
			message +="#";
	}
	client.println(message);
	start_write = false;
}

String read_temperature(){
	// ******************************************************* //
	// Enter your own code in the read function if you consider
	// to obtain 'val' value to return
	// ******************************************************* //
	float val_temperature = dht_temperature.readTemperature();
	return String(val_temperature);
}

